#include "range_definition.h"

Range_Boundary::Range_Boundary(int x, int y, bool left)
  : Fl_Box(FL_ROUND_UP_BOX, x, y, 30, 30, left ? "@DnArrow" : "@4UpArrow") {
}

int Range_Boundary::edge() {
  return x()+offset;
}

void Right_Boundary::drag(int dx) {
  pos_set(dx+x());
  *pos = Range_Boundary::edge();
}

void Left_Boundary::drag(int dx) {
  pos_set(dx+x());
  *pos = Range_Boundary::edge();
}

Right_Boundary::Right_Boundary(int x, int y, int mx, int *p, int *m)
  : Range_Boundary(x, y, false) {
  pos = p;
  *pos = Range_Boundary::edge();
  mate = m;
  max = mx;
  offset = 0;
}

Left_Boundary::Left_Boundary(int x, int y, int mn, int *p, int *m)
  : Range_Boundary(x, y, true) {
  pos = p;
  *pos = Range_Boundary::edge();
  mate = m;
  min = mn;
  offset = 30;
}


Trough::Trough(int x, int y, int width)
  : Fl_Box(FL_DOWN_BOX, x, y, width, 10, "") {
}

Range_Definition::Range_Definition(int x, int y, int width, int *l, int *r,
				   unsigned size)
  : Fl_Group(x, y, width, 30) {
  box(FL_FLAT_BOX);
  trough      = new Trough(x+15, y+10, width-30);
  trough->left = range_start =
    new Left_Boundary(x, y, x, l, r);
  trough->right = range_end =
    new Right_Boundary(x+width-30, y, x+width-30, r, l);
  end();
}

void Left_Boundary::pos_set(int _x) {
   position(clip_range(_x, min, (*mate)-30), y());
}
int Trough::handle(int event) {
  static int button, position;
  switch (event) {
  case FL_RELEASE:
    button = 0;
    position = 0;
  case FL_PUSH:
    button = Fl::event_button();
    position = Fl::event_x();
  case FL_DRAG:
    if(button==1 || button==2) {
      left->drag(Fl::event_x()-position);
      left->redraw();
    }
    if(button==2 || button==3) {
      right->drag(Fl::event_x()-position);
      right->redraw();
    }
    clip(Fl::event_x(), position, y(), y(), 30, 30);
    redraw();
    fl_pop_clip();
    parent()->redraw();
    position = Fl::event_x();
    return 1;
  }
  return 0;
}


int Left_Boundary::handle(int event) {
  switch (event) {
  case FL_PUSH:
  case FL_RELEASE:
  case FL_DRAG:
    pos_set(Fl::event_x()-15);
    redraw();
    parent()->redraw();
    *pos = Range_Boundary::edge();
    return 1;
  }
  return 0;
}

void Right_Boundary::pos_set(int _x) {
  position(clip_range(_x, *mate, max), y());
}

int Right_Boundary::handle(int event) {
  switch (event) {
  case FL_PUSH:
  case FL_RELEASE:
  case FL_DRAG:
    pos_set(Fl::event_x()-15);
    redraw();
    parent()->redraw();
    *pos = Range_Boundary::edge();
    return 1;
  }
  return 0;
}

