#ifndef range_definition_h
#define range_definition_h

#include <FL/Fl_Group.H>
#include <FL/Fl_Box.H>
#include <FL/Fl.H>

#include "util.h"

#define clip_range(X, MN, MX) (X < MN) ? MN : ((X > MX) ? MX : X)

class Range_Boundary : public Fl_Box {
public:
  Range_Boundary(int x, int y, bool left);
  int *mate;
  int *pos;
  int edge();
  int offset;
};

class Left_Boundary : public Range_Boundary {
public:
  Left_Boundary(int x, int y, int mn, int *p, int *m);
  int handle(int event);
  void drag(int dx);
private:
  int edge();
  int min;
  void pos_set(int x);
};

class Right_Boundary : public Range_Boundary {
public:
  Right_Boundary(int x, int y, int mx, int *p, int *m);
  int handle(int event);
  void drag(int dx);
private:
  int edge();
  int max;
  void pos_set(int x);
};

class Trough : public Fl_Box {
public:
  Trough(int x, int y, int width);
  int handle(int event);
  Right_Boundary *right;
  Left_Boundary *left;
};

class Range_Definition : public Fl_Group {
 public:
  Range_Definition(int x, int y, int width, int * l, int *r, unsigned size);
 private:
  Left_Boundary *range_start;
  Right_Boundary *range_end;
  Trough *trough;
};



#endif
