#include "synth.h"

jack_port_t *output_port_l;
jack_port_t *output_port_r;
jack_port_t *mod_out_l;
jack_port_t *mod_out_r;
jack_port_t *input_port_l;
jack_port_t *input_port_r;

unsigned sr;

inline float phase(float value, float increment, unsigned max) {
  return fmod((value+increment), max);
}

inline float interp(float index, float *buf, unsigned bufsize,
		    unsigned points) {
  float frac, a, b, c, d, cminusb, *fp;
  int trunc;

  switch (points) {
  case 0:
  case 1:
    return buf[(int)index];
  case 2:
    trunc = floorf(index);
    frac = index - trunc;
    a = buf[(int) trunc] * (1 - frac);
    b = buf[(int) ceilf(index)] * frac;
    return a+b;
  case 4:
    trunc = (int) index;
    frac = index - trunc;
    a = buf[(trunc-1) % bufsize];
    b = buf[trunc];
    c = buf[(trunc + 1) % bufsize];
    d = buf[(trunc + 2) % bufsize];
    cminusb = c - b;
    return b + frac * (cminusb - 0.1666667f * (1.f-frac) *
		       ((d - a - 3.f * cminusb) * frac +
			(d + 2.f * a - 3.f * b)));
  }
}

int process (jack_nframes_t nframes, void *unused) {
  static float phase_wave_l = 0.0f;
  static float phase_wave_r = 0.0f;
  static float phase_control_l = 0.0f;
  static float phase_control_r = 0.0f;
  unsigned i;
  jack_default_audio_sample_t *out_buf_l =
    jack_port_get_buffer(output_port_l, nframes);
  jack_default_audio_sample_t *out_mod_l =
    jack_port_get_buffer(mod_out_l, nframes);
  jack_default_audio_sample_t *out_buf_r =
    jack_port_get_buffer(output_port_r, nframes);
  jack_default_audio_sample_t *out_mod_r =
    jack_port_get_buffer(mod_out_r, nframes);
  if (synth_data->freeze == 1) {
    return 0;
  } else {
    if (synth_data->mute == 1) {
      for(i = 0; i < nframes; i++) {
	out_buf_l[i] = out_buf_r[i] = out_mod_l[i] = out_mod_r[i] = 0;
      }
    } else {
      {
	jack_default_audio_sample_t *in_buf_l =
	  jack_port_get_buffer(input_port_l, nframes);
	jack_default_audio_sample_t *in_buf_r =
	  jack_port_get_buffer(input_port_r, nframes);
	float speed_l, mod_l, speed_r, mod_r;
	float hz_inc_l =
	  (float)Waveforms[0].size/(float)sr;
	float hz_inc_r =
	  (float)Waveforms[1].size/(float)sr;
	float hz_inc_l_con =
	  (float)Waveforms[2].size/(float)sr;
	float hz_inc_r_con =
	  (float) Waveforms[3].size/(float)sr;
	for(i = 0; i < nframes; i++) {
	  mod_l = in_buf_l[i]*synth_data->left.range;
	  if(synth_data->left.control) {
	    speed_l = in_buf_l[i] * synth_data->left.speed_range;
	    speed_l += synth_data->left.speed_center;
	    phase_control_l = phase(phase_control_l, speed_l*hz_inc_l_con,
				    Waveforms[2].size);
	    speed_l =
	      interp(phase_control_l, Waveforms[2].data,
		     Waveforms[2].size,
		     synth_data->interp);
	    speed_l *= synth_data->left.control_range;
	    mod_l += speed_l;
	  }
	  mod_l += synth_data->left.center;
	  
	  mod_r = in_buf_r[i] * synth_data->right.range;
	  if(synth_data->right.control) {
	    speed_r = in_buf_r[i] * synth_data->right.speed_range;
	    speed_r += synth_data->right.speed_center;
	    phase_control_r = phase(phase_control_r, speed_r*hz_inc_r_con,
				    Waveforms[3].size);
	    speed_r =
	      interp(phase_control_r, Waveforms[3].data,
		     Waveforms[3].size,
		     synth_data->interp);
	    speed_r *= synth_data->right.control_range;
	    mod_r += speed_r;

	  }
	  mod_r += synth_data->right.center;

	  phase_wave_l = phase(phase_wave_l, mod_l*hz_inc_l,
			       Waveforms[0].size);
	  phase_wave_r = phase(phase_wave_r, mod_r*hz_inc_r,
			       Waveforms[1].size);
	  out_mod_l[i] =
	    interp(phase_wave_l, Waveforms[0].data,
		   Waveforms[0].size,
		   synth_data->interp);
	  out_mod_r[i] =
	    interp(phase_wave_r, Waveforms[1].data,
		   Waveforms[1].size,
		   synth_data->interp);
	  out_buf_l[i] = out_mod_l[i] *
	    synth_data->left.amp * synth_data->master *
	    abs(1.0-synth_data->left.mute);
	  out_buf_r[i] = out_mod_r[i] *
	    synth_data->right.amp * synth_data->master *
	    abs(1.0-synth_data->right.mute);
	}
      }
      return 0;
    }
  }
}

int jack_us(const char *name) {
  jack_client_t *client;

  if(!(client = jack_client_open(name, JackNoStartServer, NULL))) {
      fprintf(stderr, "%s: could not connect to jack\n", name);
      return 1;
  }
  sr = jack_get_sample_rate(client);
  output_port_l =
  jack_port_register(client, "output_l", JACK_DEFAULT_AUDIO_TYPE,
		     JackPortIsOutput, 0);
  output_port_r =
  jack_port_register(client, "output_r", JACK_DEFAULT_AUDIO_TYPE,
		     JackPortIsOutput, 0);
  mod_out_l =
    jack_port_register(client, "mod_out_l", JACK_DEFAULT_AUDIO_TYPE,
		       JackPortIsOutput, 0);
  mod_out_r =
    jack_port_register(client, "mod_out_r", JACK_DEFAULT_AUDIO_TYPE,
		       JackPortIsOutput, 0);
  input_port_l =
  jack_port_register(client, "input_l", JACK_DEFAULT_AUDIO_TYPE,
		     JackPortIsInput, 0);
  input_port_r =
  jack_port_register(client, "input_r", JACK_DEFAULT_AUDIO_TYPE,
		     JackPortIsInput, 0);
  jack_set_process_callback(client, process, 0);
  if (jack_activate(client)) {
    fprintf(stderr, "%s: cannot activate client\n", name);
    return 2;
  }
}
